EnableExplicit

LibBuild_Include(UnHex)
LibBuild_Include(GetTickCountAPI)

;Enumeration WinSock_WSRACFlags
  #WinSock_WSRACFlags_ReceiveSizeOnly = 1
 ; EndEnumeration

  Structure WinSock_HTTPReceiverBuffer
    hSocket.i
    UsedBuffer.i
    SizeBuffer.i
    ptrBuffer.i
    LastData.i
    KnownHSize.i
    KnownSize.i
    KnownNextChunk.i
    KnownNextData.i
    ChunkQuTo.i
    ChunkQuFrom.i
    ChunkQuLen.i
    Flags.i
    TimeOut.i
    EndStructure
  #WinSock_HTTPReceiverBufferInitSize = 1024 * 1024
  #WinSock_HTTPReceiverMaxHeaderSize = 1024 * 32

Procedure.i WinSock_HTTPReceiver_Create(hSocket.i, Flags.i = 0, TimeOut.i = 4000)
  Define *xType1.WinSock_HTTPReceiverBuffer

  If hSocket > 0
    *xType1 = MemAlloc(SizeOf(WinSock_HTTPReceiverBuffer))
    If *xType1
      *xType1\ptrBuffer = MemAlloc(#WinSock_HTTPReceiverBufferInitSize)
      If *xType1\ptrBuffer
        *xType1\Flags = Flags
        *xType1\LastData = GetTickTimeMS()
        *xType1\SizeBuffer = #WinSock_HTTPReceiverBufferInitSize
        *xType1\TimeOut = TimeOut
        *xType1\hSocket = hSocket
        *xType1\KnownSize = -2
        ProcedureReturn *xType1
        EndIf
      MemFree(*xType1)
      EndIf
    closesocket_(hSocket)
    EndIf
  EndProcedure

;' ReturnSize: -2 = unknown / error, -1 = unknown (chunked / not provided by server), -x = future errors?
Procedure.i WinSock_HTTPReceiver_GetStatus(hReceiver.i, *ReturnProgress, *ReturnSize)
  Define.i xLong1
  Define.i xLong2
  Define.i xLong3
  Define.i xLong4
  Define.s xChar1
  Define.i xSizeCurCheckHead
  Define.i xFoundHead
  Define.WinSock_HTTPReceiverBuffer xType1
  Define.i Result
  #Net_Max_Buffer_Length = 4 * 8192

  If hReceiver
    MemCopyRTL(@xType1, hReceiver, SizeOf(WinSock_HTTPReceiverBuffer))
    If xType1\hSocket > 0
      If xType1\UsedBuffer + #Net_Max_Buffer_Length > xType1\SizeBuffer
        xType1\SizeBuffer = xType1\SizeBuffer * 2 + #Net_Max_Buffer_Length
        xLong1 = MemAlloc(xType1\SizeBuffer)
        If xLong1 = 0
          Goto Failure
          EndIf
        MemCopyRTL(xLong1, xType1\ptrBuffer, xType1\UsedBuffer)
        MemFree(xType1\ptrBuffer)
        xType1\ptrBuffer = xLong1
        EndIf
  
      xLong1 = xType1\ptrBuffer
      xLong1 + xType1\UsedBuffer
      xLong1 = recv_(xType1\hSocket, xLong1, #Net_Max_Buffer_Length, 0)
      
      If xLong1 = -1
        xLong1 = WSAGetLastError_()
        If xLong1 = 0
          Goto TimeOutCheck
          ElseIf xLong1 = 10036
          Goto TimeOutCheck
          ElseIf xLong1 = 10035
          Goto TimeOutCheck
          EndIf
        Goto Failure
        ;Case 10036 'WSAEINPROGRESS
        ;Case 10035 'WSAEWOULDBLOCK
        ;Case 0
        ElseIf xLong1 = 0 ;, 10036, 10035
          ;TimeOut
TimeOutCheck:
          If IsTickTimeElapsedMS(xType1\LastData, xType1\TimeOut)
            Goto TimedOut
            EndIf
        ElseIf xLong1 > 0
        xType1\UsedBuffer = xType1\UsedBuffer + xLong1
        xType1\LastData = GetTickTimeMS()
        If xType1\KnownHSize = 0
          xSizeCurCheckHead = xType1\UsedBuffer
          If xSizeCurCheckHead > #WinSock_HTTPReceiverMaxHeaderSize
            xSizeCurCheckHead = #WinSock_HTTPReceiverMaxHeaderSize
            EndIf
;Debug "ParseForHead: " + Str(xSizeCurCheckHead) + "/" + Str(#WinSock_HTTPReceiverMaxHeaderSize)
          xChar1 = PeekS(xType1\ptrBuffer, xSizeCurCheckHead, #PB_Ascii)
          xLong1 = FindString(xChar1, #CRLF$)
          If xLong1
            xLong2 = FindString(xChar1, " ")
            If xLong2 < xLong1 And xLong2
              xLong2 = Val(Trim(Mid(xChar1, xLong2 + 1, xLong1 - xLong2 - 1)))
              If Not xLong2 = 200
                Goto Failure
                EndIf
              EndIf
            xFoundHead = FindString(xChar1, #CRLF$ + #CRLF$)
            If xFoundHead
;Debug "HeadOK"
              xChar1 = Left(xChar1, xFoundHead + 1)
              xLong2 = FindString(xChar1, #CRLF$ + "Content-Length: ", 1, #PB_String_NoCase)
              If xLong2
;Debug "FoundCL"
                xLong3 = FindString(xChar1, #CRLF$, xLong2 + 1)
                If xLong3 = 0
;Debug "Fail1"
                  Goto Failure
                  EndIf
                xLong2 = Val(Trim(Mid(xChar1, xLong2 + 18, xLong3 - xLong2 - 18)))
                xType1\KnownHSize = xFoundHead + 3
                xType1\KnownSize = xLong2 + xFoundHead + 3
                If (xType1\Flags & #WinSock_WSRACFlags_ReceiveSizeOnly)
                  Goto LengthSuccess
                  EndIf
                Else
                If FindString(xChar1, "Transfer-Encoding: chunked", 1, #PB_String_NoCase) = 0
                  Goto Failure
                  EndIf
                xType1\KnownSize = -1
                xType1\KnownNextChunk = xFoundHead + 3
                xType1\KnownNextData = xFoundHead + 3
                xType1\KnownHSize = xFoundHead + 3
                If (xType1\Flags & #WinSock_WSRACFlags_ReceiveSizeOnly)
                  Goto LengthSuccess
                  EndIf
                EndIf
              ;If xType1\KnownHSize = 0 Then GoTo Incomplete
              EndIf
            EndIf
          If xSizeCurCheckHead = #WinSock_HTTPReceiverMaxHeaderSize ;And xFoundHead = 0
;Debug "HeadFail1"
            Goto Failure
            EndIf
          EndIf
        If xType1\KnownHSize
          If xType1\KnownNextChunk = 0
            If xType1\KnownSize = xType1\UsedBuffer
              Goto Success
              EndIf
            Else
TryNextChunk:
;Debug "TryNextChunk"
            xLong1 = xType1\UsedBuffer - xType1\KnownNextChunk
            If xLong1 > 2
              If xLong1 > 64
                xLong1 = 64
                EndIf
              ;xChar1 = String(xLong1, vbNullChar)
              xLong2 = xType1\ptrBuffer
              xLong2 + xType1\KnownNextChunk
              ;MemCopyRTL(@xChar1, xLong2, xLong1)
              xChar1 = PeekS(xLong2, xLong1, #PB_Ascii)
;Debug "CheckIn: " + xChar1
              xLong1 = FindString(xChar1, #CRLF$)
              If xLong1
                xLong2 = UnHex(Left(xChar1, xLong1 - 1))
;Debug "NextChunk: " + Str(xLong2)
                If xType1\ChunkQuLen
;                    Stop
                  xType1\ChunkQuTo + xType1\ptrBuffer
                  xType1\ChunkQuFrom + xType1\ptrBuffer
                  MemCopyRTL(xType1\ChunkQuTo, xType1\ChunkQuFrom, xType1\ChunkQuLen)
                  EndIf
                If xLong2 = 0
                  Goto Success
                  EndIf
                ;xType1\KnownNextChunk = xType1\KnownNextChunk
                xType1\ChunkQuTo = xType1\KnownNextData
                xType1\ChunkQuFrom = xType1\KnownNextChunk + xLong1 + 1
                xType1\ChunkQuLen = xLong2
                xType1\KnownNextData = xType1\KnownNextData + xLong2
                xType1\KnownNextChunk = xType1\KnownNextChunk + xLong2 + xLong1 + 3
                Goto TryNextChunk
                EndIf
              EndIf
            EndIf
          EndIf
        Else
Failure:
LengthSuccess:
TimedOut:
        MemFree(xType1\ptrBuffer)
        xType1\ptrBuffer = 0
Success:
        Result = #True
        closesocket_(xType1\hSocket)
        xType1\hSocket = 0
        EndIf
      MemCopyRTL(hReceiver, @xType1, SizeOf(WinSock_HTTPReceiverBuffer))
      Else
      Result = #True
      EndIf
    If *ReturnProgress
      PokeI(*ReturnProgress, xType1\UsedBuffer)
      EndIf
    If *ReturnSize
      PokeI(*ReturnSize, xType1\KnownSize)
      EndIf
    Else
    Result = #True
    EndIf
  ProcedureReturn Result
  EndProcedure

Procedure WinSock_HTTPReceiver_Destroy(*hReceiver.WinSock_HTTPReceiverBuffer)
  If *hReceiver
    If *hReceiver\hSocket > 0
      closesocket_(*hReceiver\hSocket)
      EndIf
    If *hReceiver\ptrBuffer
      MemFree(*hReceiver\ptrBuffer)
      EndIf
    MemFree(*hReceiver)
    EndIf
  EndProcedure

Procedure.i WinSock_HTTPReceiver_GetCompleteDataPos(*hReceiver.WinSock_HTTPReceiverBuffer, *Len)
  Define.i xLong1
  Define.i Result

  If *hReceiver
    If *hReceiver\hSocket = 0
      If *hReceiver\ptrBuffer
        If *hReceiver\KnownNextData
          *hReceiver\UsedBuffer = *hReceiver\KnownNextData
          *hReceiver\KnownNextData = 0;;; new!!!?!!!
          EndIf
        ;*hReceiver\KnownHSize = 0
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+40, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+80, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+120, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+160, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+200, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+240, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+280, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+320, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+360, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+400, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+440, 256, #PB_Ascii)
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer+480, 256, #PB_Ascii)
;Debug *hReceiver\KnownHSize
;Debug "Downloaded: " + PeekS(*hReceiver\ptrBuffer + *hReceiver\KnownHSize, 256, #PB_Ascii)
        xLong1 = *hReceiver\UsedBuffer - *hReceiver\KnownHSize
        Result = *hReceiver\ptrBuffer + *hReceiver\KnownHSize
        PokeI(*Len, xLong1)
        EndIf
      EndIf
    EndIf
  ProcedureReturn Result
  EndProcedure

; IDE Options = PureBasic 5.21 LTS (Windows - x86)
; CursorPosition = 28
; FirstLine = 21
; Folding = -
; EnableUnicode
; EnableXP