﻿EnableExplicit

#FILE_WRITE     = $40000000
Macro CreateFile_CreateWO(Filename)
  CreateFile_(Filename, #FILE_WRITE, $1, 0, #CREATE_ALWAYS, $20, 0)
  EndMacro

CompilerIf #PB_Compiler_Debugger
  Macro PrintNX(xText)
    PrintN(xText)
    Debug xText
    EndMacro
  CompilerElse
  Macro PrintNX(xText)
    PrintN(xText)
    EndMacro
  CompilerEndIf


Define.s xChar1
Define.s xChar2
Define.i hDC
Define.i hMeta
;Define.RECT xType1
Define.i xSize
Define *Mem
Define.i hFile
Define.i xLong1
Define.i xResult

OpenConsole()
PrintNX("Emf to Wmf converter 0.1")
PrintNX("Copyright 2014 by Jan Vorel")

xResult = 1 ; Parameters
        ; 2 ; Initialization
        ; 3 ; Loading
        ; 4 ; Saving
        ; 0 ; Success
xChar1 = ProgramParameter()
xChar2 = ProgramParameter()

PrintNX("")
If Len(ProgramParameter()) Or Not Len(xChar1)
  PrintNX("Syntax: Emf2wmf <Source/EMFFile> [Destination/WMFFilename]")
  ElseIf LCase(Right(xChar1, 4)) = ".wmf"
  PrintNX("Source file must not have .wmf extension")
  ElseIf LCase(xChar1) = LCase(xChar2)
  PrintNX("Source and destination must not be the same")
  ElseIf LCase(Right(xChar2, 4)) = ".wmf"
  PrintNX("Destination file must not have .emf extension")
  Else
  If Not Len(xChar2)
    xChar2 = GetPathPart(xChar1) + GetFilePart(xChar1, #PB_FileSystem_NoExtension) + ".wmf"
;    PrintNX("Output file is " + xChar2)
    EndIf
  hDC = GetDC_(0)
  If hDC
    If GetDeviceCaps_(hDC, 12) < 16
      PrintNX("Non-Fullcolor display, untestet, please report if working!")
      EndIf
    xResult = 3
    PrintNX("Loading source file: " + Chr(34) + xChar1 + Chr(34) + "...")
    hMeta = GetEnhMetaFile_(@xChar1)
    If hMeta
      xSize = GetWinMetaFileBits_(hMeta, 0, 0, #MM_ANISOTROPIC, hDC)
      If xSize
        *Mem = LocalAlloc_(0, xSize)
        If *Mem
          If GetWinMetaFileBits_(hMeta, xSize, *Mem, #MM_ANISOTROPIC, hDC) = xSize
            xResult = 4
            PrintNX("Saving to destination file: " + Chr(34) + xChar2 + Chr(34) + "...")
            hFile = CreateFile_CreateWO(@xChar2)
            If hFile = -1
              PrintNX("  could not create output file (CreateFile)")
              Else
              If WriteFile_(hFile, *Mem, xSize, @xLong1, 0)
                If xSize = xLong1
                  xResult = 0
                  PrintNX("  conversion successful.")
                  EndIf
                EndIf
              CloseHandle_(hFile)
              If xResult
                PrintNX("  failed to write data to file (WriteFile)")
                DeleteFile_(@xChar2)
                EndIf
              EndIf
            Else
            PrintNX("  could not process EMF file (GetWinMetaFileBits/2)")
            EndIf
          LocalFree_(*Mem)
          Else
          PrintNX("  could not allocate " + Str(xSize) + " bytes (LocalAlloc)")
          EndIf
        Else
        PrintNX("  could not process EMF file (GetWinMetaFileBits/1)")
        EndIf
      DeleteEnhMetaFile_(hMeta)
      Else
      PrintNX("  could not load EMF file (GetEnhMetaFile)")
      EndIf
    ReleaseDC_(0, hDC)
    Else
    PrintNX("Initialization Error (GetDC)")
    xResult = 2
    EndIf
  EndIf
CloseConsole()
End xResult

; IDE Options = PureBasic 5.21 LTS (Windows - x64)
; ExecutableFormat = Console
; CursorPosition = 98
; FirstLine = 82
; Folding = -
; Executable = emf2wmf.exe
; DisableDebugger
; IncludeVersionInfo
; VersionField0 = 0,1,0,0
; VersionField1 = 0,1,0,0
; VersionField2 = ctuser
; VersionField3 = Emf to Wmf converter
; VersionField4 = 0.1
; VersionField5 = 0.1
; VersionField6 = Emf to Wmf converter
; VersionField7 = emf2wmf
; VersionField8 = emf2wmf.exe
; VersionField9 = Copyright 2014 by Jan Vorel
; VersionField13 = info@ctuser.net
; VersionField14 = http://ctuser.net